//Author:    Timothy Pearson <kb9vqf@pearsoncomputing.net>, (C) 2012
//Copyright: See COPYING file that comes with this distribution

#ifndef REMOTELAB_SCOPEPART_H
#define REMOTELAB_SCOPEPART_H

#include <tdeparts/browserextension.h>
#include <tdeparts/statusbarextension.h>
#include <tdeparts/part.h>
#include <kurl.h>

#include <tqtrla.h>

#define MAXTRACES 255

class TDEAboutData;
using KParts::StatusBarExtension;
class TraceWidget;
class TQSocket;
class TQTimer;
class TQMutex;
class TQRectF;
class TQGridLayout;
class TQCheckBox;
class TQGroupBox;
class ScopeBase;

namespace RemoteLab
{
	class TraceControlWidget : public TQWidget
	{
		Q_OBJECT

		public:
			TraceControlWidget(TQWidget *parent=0, const char *name=0);
			~TraceControlWidget();

		public:
			void setVoltsPerDivList(TQDoubleList list);
			void setSelectedVoltsPerDiv(double vdiv);
			void setTraceEnabled(bool enabled);
			void setTraceName(TQString name);

		signals:
			void enableChanged(bool enabled);
			void voltsPerDivChanged(double vdiv);

		private slots:
			void enableClicked();
			void vdivChanged(int index);

		private:
			TQGroupBox* m_groupBox;
			TQGridLayout* m_primaryLayout;
			TQComboBox* m_voltsDivComboBox;
			TQCheckBox* m_channelEnabledCheckBox;

			TQDoubleList m_voltsDivList;
	};

	class ScopePart : public KParts::RemoteInstrumentPart
	{
		Q_OBJECT
	
		public:
			ScopePart( QWidget *, const char *, TQObject *, const char *, const TQStringList&);
			~ScopePart();
	
			virtual bool openFile() { return false; }	// pure virtual in the base class
			virtual bool closeURL();
			static TDEAboutData *createAboutData();

		public slots:
			virtual bool openURL(const KURL &url);
			void updateZoomWidgetLimits(const TQRectF& zoomRect);
		
		private slots:
			void postInit();
			void processLockouts();
			void updateGraticule();
			void connectionFinishedCallback();
			void disconnectFromServerCallback();
			void connectionStatusChangedCallback();
			void setTickerMessage(TQString message);
			void mainEventLoop();
			void startDAQ();
			void stopDAQ();
			void updateTraceControlWidgets();
			void traceControlEnableChanged(bool enabled);
			void traceControlVDivChanged(double vdiv);
			void cursorLevelChanged(uint cursor, double level);
			void startScope();
			void stopScope();
			void saveWaveforms();
			void recallWaveforms();
			virtual void postProcessTrace();
	
		private:
			TraceWidget* m_traceWidget;
			TQGridLayout* m_traceControlWidgetGrid;
			int m_commHandlerState;
			int m_commHandlerMode;
			int m_commHandlerCommandState;
			TQTimer* m_forcedUpdateTimer;
			TQTimer* m_updateTimeoutTimer;
			bool m_connectionActiveAndValid;
			unsigned char m_tickerState;
			TQ_INT16 m_maxNumberOfTraces;
			TQ_INT32 m_currentOpChannel;
			TQ_INT32 m_nextOpChannel;
			TQ_INT16 m_nextOpParameter16;
			TQ_INT16 m_hdivs;
			TQ_INT16 m_vdivs;
			TQ_INT16 m_triggerChannel;
			bool m_running;
			double m_triggerLevel;
			TQ_INT32 m_samplesInTrace[MAXTRACES+1];
			bool m_channelActive[MAXTRACES+1];
			double m_voltsDiv[MAXTRACES+1];
			double m_secsDiv[MAXTRACES+1];
			TraceControlWidget* m_traceControlWidgetList[MAXTRACES];
			bool m_triggerLevelSet;
			bool m_runningSet;
			bool m_voltsDivSet[MAXTRACES+1];
			bool m_channelActiveSet[MAXTRACES+1];
			bool m_lastChangesRequireFullUpdate;
			ScopeBase* m_base;
			TQMutex* m_instrumentMutex;
			bool stopTraceUpdate;
	};
}

#endif
